from ftw.colorbox.interfaces import IColorboxSettings
from plone.registry.interfaces import IRegistry
from Products.Five.browser import BrowserView
from zope.component import getUtility


# Generating JavaScript is not sustainable especially when we are now using AMD
# patterns. So we inject a data element into the body tag for Plone 5.
# Unfortunately this is not easy to replicate in Plone 4, so we leave the JS
# generation as is, knowing that Plone 4 isn't supported forever anyway.

class SettingsInjector(object):
    """
    Injects colorbox settings into the body tag borrowing technology from patternslib
    """

    def __init__(self, context, request, field):
        self.request = request
        self.context = context
        self.field = field

    def __call__(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(IColorboxSettings)
        # Admittably horrible, but this is what we get from settings
        jsonized = "{%s}" % ",".join(settings.colorbox_config).replace('\'', '"')
        data = {
            'data-ftw-colorbox-options': jsonized
        }
        return(data)


# Plone 4 ONLY - this should be considered deprecated (see comment above)

class InitColorBoxView(BrowserView):

    def __call__(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(IColorboxSettings)
        exclude_expression = settings.colorbox_exclude
        return """
            var ftwColorboxInitialize = function() {
                var staticOptions = {
                    'title': function(){
                        var link = $(this);
                        var title = link.attr('title');

                        // Use the value from the attrib "data-caption" if there is one.
                        var caption = link.data('caption');
                        if (caption) {
                            title = caption;
                        }

                        return title;
                    }
                }
                var options = {%s}  // The options stored in the Plone registry.
                $.extend(options, staticOptions)
                $('a.colorboxLink').not(function(){%s}).colorbox(options);
            }
            jQuery(function($) {
                ftwColorboxInitialize();
            });
        """ % (','.join(settings.colorbox_config), exclude_expression)



from ftw.upgrade import UpgradeStep
from Products.CMFPlone.utils import getFSVersionTuple


IS_PLONE_4 = getFSVersionTuple() < (5, )


class RemoveColorboxJsFromPlone4(UpgradeStep):
    """Remove redundant colorbox.js from Plone 4.
    """

    def __call__(self):
        if IS_PLONE_4:
            self.install_upgrade_profile()



from ftw.colorbox import colorboxMessageFactory as _
from zope import schema
from zope.interface import Interface


class IColorboxSettings(Interface):
    """Interface for registry entries.
    """
    image_size = schema.Text(
        title=_(u'label_imagesize', default=u'Image size'),
        description=_(u'help_imagesize',
            default=u'Provide the name of an image size configured in the '
            'imaging control panel (e.g. large). If no size is given images '
            'will be displayed in original size.'),
        default=u'',
        required=False,
    )

    colorbox_config = schema.List(
        title=_(u'label_colorboxconfig', default=u'Colorbox config'),
        description=_(u'help_colorboxconfig',
            default=u'One item per line. For example speed:500.'),
        value_type=schema.TextLine(),
        default=[
            "'photo': true",
            "'current': '{current}/{total}'",
            "'maxWidth': '100%'",
            "'maxHeight': '100%'"])

    colorbox_exclude = schema.Text(
        title=_(u'label_colorbox_exclude', default=u'Colorbox Exclude'),
        description=_(u'help_colorbox_exlude',
            default=u'Javascript that runs inside a jquery '
                      '".not(function() { ... })" expression, where "this" is '
                      'the a.colorbox element. The expression has to '
                      'return either "true" for removing an item from the set '
                      'or "false" to keep an item in the set. An example would '
                      'bt to exlude certain classes: '
                      '"return $(this).hasClass(\'slick-cloned\');"'),
        default=u'return false;',
        required=False,
    )

    show_link = schema.Bool(
        title=_(u'label_showlink', default=u'Show link'),
        description=_(u'help_showlink',
            default=u'If enabled, a link to download the original image is '
                     'displayed in the overlay.'),
        default=False)

    row_size = schema.Int(
        title=_(u'label_row_size', default=u'Row Size'),
        description=_(u'help_row_size',
            default=u'The number of images shown per row. If set to 0 all '
                     'images are floated in the same row.'),
        default=0
    )

    batch_size = schema.Int(
        title=_(u'label_batch_size', default=u'Batch Size'),
        description=_(u'help_batch_size',
            default=u'The number of thumbnails shown on a page.'),
        default=12
    )



from ftw.upgrade import UpgradeStep
from Products.CMFPlone.utils import getFSVersionTuple


IS_PLONE_5 = getFSVersionTuple() >= (5, )


class ReconfigureBundlingOnPlone5(UpgradeStep):
    """Reconfigure bundling on Plone5 to use Yarn.
    """

    def __call__(self):
        if IS_PLONE_5:
            self.install_upgrade_profile()



from ftw.upgrade import UpgradeStep
from plone import api
import pkg_resources


IS_PLONE_5 = pkg_resources.get_distribution('Products.CMFPlone').version >= '5'


class MoveResourcesFromLegacyBundleToItsOwn(UpgradeStep):
    """Move resources from legacy bundle to its own.
    """

    def __call__(self):
        if IS_PLONE_5:
            to_remove = [u'resource-ftw-colorbox-resources-jquery-colorbox',
                         u'init-colorbox-js',
                         u'resource-ftw-colorbox-resources-colorbox-js',
                         u'resource-ftw-colorbox-resources-colorbox-css']

            record = 'plone.bundles/plone-legacy.resources'
            resources = api.portal.get_registry_record(record)

            for resource in to_remove:
                if resource in resources:
                    resources.remove(resource)

            self.install_upgrade_profile()
        else:
            # Do nothing on plone 4
            pass



from setuptools import setup, find_packages
import os

version = '1.5.0'

tests_require = [
    'ftw.testbrowser',
    'ftw.testing',
    'plone.app.testing',
]

setup(
    name='ftw.colorbox',
    version=version,
    description="An image gallery for Plone using ColorBox",
    long_description='{0}\n{1}'.format(
        open("README.rst").read(),
        open(os.path.join("docs", "HISTORY.txt")).read()
    ),

    classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.3',
        'Framework :: Plone :: 5.1',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],

    keywords='ftw colorbox',
    author='4teamwork AG',
    author_email='mailto:info@4teamwork.ch',
    url='https://github.com/4teamwork/ftw.colorbox',
    license='GPL2',

    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['ftw'],
    include_package_data=True,
    zip_safe=False,

    install_requires=[
        'ftw.upgrade',
        'setuptools',
        'Plone',
        'plone.api',
    ],

    tests_require=tests_require,
    extras_require=dict(tests=tests_require),

    entry_points="""
    # -*- Entry points: -*-
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



from ftw.testing.genericsetup import GenericSetupUninstallMixin
from ftw.testing.genericsetup import apply_generic_setup_layer
from unittest import TestCase


@apply_generic_setup_layer
class TestGenericSetupUninstall(TestCase, GenericSetupUninstallMixin):
    package = 'ftw.colorbox'
    # FIXME - we shouldn't need to have to do this
    skip_files = ('registry.xml',)
